@@ -13,8 +13,7 @@
 #  GNU General Public License for more details.
 #
 #  You should have received a copy of the GNU General Public License
-#  along with this program; if not, write to the Free Software
-#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#  along with this program; if not, see <http://www.gnu.org/licenses/>.
 #
 ################################################################################
 import re
@@ -42,6 +41,7 @@ class FrameException(Error):
 TITLE_FID          = "TIT2"
 SUBTITLE_FID       = "TIT3"
 ARTIST_FID         = "TPE1"
+ALBUM_ARTIST_FID   = "TPE2"
 ALBUM_FID          = "TALB"
 TRACKNUM_FID       = "TRCK"
 GENRE_FID          = "TCON"
@@ -298,7 +298,6 @@ class TextFrame(Frame):
 
 
 class UserTextFrame(TextFrame):
-
     @requireUnicode("description", "text")
     def __init__(self, id=USERTEXT_FID, description=u"", text=u""):
         super(UserTextFrame, self).__init__(id, text=text)
@@ -712,6 +711,15 @@ class ImageFrame(Frame):
         else:
             raise ValueError("Invalid APIC picture type: %s" % s)
 
+    def makeFileName(self, name=None):
+        name = ImageFrame.picTypeToString(self.picture_type) if not name \
+                                                             else name
+        ext = self.mime_type.split("/")[1]
+        if ext == "jpeg":
+            ext = "jpg"
+        name = '.'.join([name, ext])
+        return name
+
 
 class ObjectFrame(Frame):
 
@@ -748,11 +756,11 @@ class ObjectFrame(Frame):
 
         Data string format:
         <Header for 'General encapsulated object', ID: "GEOB">
-         Text encoding          $xx
-         MIME type              <text string> $00
-         Filename               <text string according to encoding> $00 (00)
-         Content description    <text string according to encoding> $00 (00)
-         Encapsulated object    <binary data>
+        Text encoding          $xx
+        MIME type              <text string> $00
+        Filename               <text string according to encoding> $00 (00)
+        Content description    <text string according to encoding> $00 (00)
+        Encapsulated object    <binary data>
         '''
         super(ObjectFrame, self).parse(data, frame_header)
 
@@ -1472,6 +1480,12 @@ def createFrame(tag_header, frame_header, data):
 def decodeUnicode(bites, encoding):
     codec = id3EncodingToString(encoding)
     log.debug("Unicode encoding: %s" % codec)
+    if (codec.startswith("utf_16") and
+            len(bites) % 2 != 0 and bites[-1] == b"\x00"):
+        # Catch and fix bad utf16 data, it is everywhere.
+        log.warning("Fixing utf16 data with extra zero bytes")
+        bites = bites[:-1]
+    # XXX: not sure if the strip is necessary since the above fix of the data.
     return unicode(bites, codec).rstrip(b"\x00")
 
 
@@ -1758,5 +1772,7 @@ NONSTANDARD_ID3_FRAMES = {
              ID3_V2, apple.TGID),
     "WFED": ("iTunes extension; podcast feed URL?",
              ID3_V2, apple.WFED),
+    "TCAT": ("iTunes extension; podcast category.",
+             ID3_V2, TextFrame),
 }
 
