@@ -13,8 +13,7 @@
 #  GNU General Public License for more details.
 #
 #  You should have received a copy of the GNU General Public License
-#  along with this program; if not, write to the Free Software
-#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#  along with this program; if not, see <http://www.gnu.org/licenses/>.
 #
 ################################################################################
 
@@ -62,8 +61,10 @@
 # OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ################################################################################
 from __future__ import print_function
+import os
 import sys
 import time
+import types
 import struct
 from ..compat import PY2
 from .. import LOCAL_ENCODING
@@ -77,28 +78,48 @@ try:
 except ImportError:
     _CAN_RESIZE_TERMINAL = False
 
-USE_ANSI = True
-'''If set to False no ANSI codes are ever used. Otherwise, use ``enableColor``
-to toggle ANSI on a per file stream basis.'''
-
-CSI = '\033['
 
 class AnsiCodes(object):
+    _USE_ANSI = False
+    _CSI = '\033['
+
     def __init__(self, codes):
         def code_to_chars(code):
-            return CSI + str(code) + 'm'
+            return AnsiCodes._CSI + str(code) + 'm'
 
         for name in dir(codes):
             if not name.startswith('_'):
                 value = getattr(codes, name)
                 setattr(self, name, code_to_chars(value))
 
-    def __getattribute__(self, name):
-        global USE_ANSI
+                # Add color function
+                for reset_name in ("RESET_%s" % name, "RESET"):
+                    if hasattr(codes, reset_name):
+                        reset_value = getattr(codes, reset_name)
+                        setattr(self, "%s" % name.lower(),
+                                AnsiCodes._mkfunc(code_to_chars(value),
+                                                  code_to_chars(reset_value)))
+                        break
 
-        name = name.upper()
+    @staticmethod
+    def _mkfunc(color, reset):
+        def _cwrap(text, *styles):
+            if not AnsiCodes._USE_ANSI:
+                return text
+
+            s = u''
+            for st in styles:
+                s += st
+            s += color + text + reset
+            if styles:
+                s += Style.RESET_ALL
+            return s
+        return _cwrap
+
+    def __getattribute__(self, name):
         attr = super(AnsiCodes, self).__getattribute__(name)
-        if attr.startswith(CSI) and not USE_ANSI:
+        if (hasattr(attr, "startswith") and attr.startswith(AnsiCodes._CSI)
+                and not AnsiCodes._USE_ANSI):
             return ''
         else:
             return attr
@@ -106,6 +127,16 @@ class AnsiCodes(object):
     def __getitem__(self, name):
         return getattr(self, name.upper())
 
+    @staticmethod
+    def init(enabled):
+        if not enabled:
+            AnsiCodes._USE_ANSI = False
+        else:
+            AnsiCodes._USE_ANSI = True
+            if "TERM" in os.environ and os.environ["TERM"] == "dumb":
+                AnsiCodes._USE_ANSI = False
+
+
 class AnsiFore:
     GREY    = 30
     RED     = 31
@@ -494,13 +525,16 @@ def _printWithColor(s, color, file):
 
 
 if __name__ == "__main__":
-    USE_ANSI = True
+    AnsiCodes.init(True)
 
     def checkCode(c):
-        return c[0] != '_' and "RESET" not in c
+        return (c[0] != '_' and
+                "RESET" not in c and
+                c[0] == c[0].upper()
+               )
 
     for bg_name, bg_code in ((c, getattr(Back, c))
-                        for c in dir(Back) if checkCode(c)):
+                             for c in dir(Back) if checkCode(c)):
         sys.stdout.write('%s%-7s%s %s ' %
                          (bg_code, bg_name, Back.RESET, bg_code))
         for fg_name, fg_code in ((c, getattr(Fore, c))
